<?php

class Block{
	private $css_class = "";
	private $css_extra_class = "";
	private $index = 0;
	private $top = 0;
	private $left = 0;
	private $prefix = "tile_";
	private $col = 0;
	private $row = 0;
	private $single = FALSE;
		
	function __construct($index) {
		$this->index = ++$index;
		
		// The structure of the image file is known
		$cols = constant("COLS_IN_IMAGE");
		$last_row_index = constant("LAST_ROW_MARKER");
		$special_group_size = constant("SPECIAL_GROUP_SIZE");
		if ( $index <= $last_row_index ){
			$this->row = ceil($index / $cols);
			$this->col = ($index % $cols) ? $index % $cols : $cols;
		} else {
			// We are at the last row - special groups
			$this->row = constant("ROWS_IN_IMAGE");;
			$this->col = $index - $last_row_index;
			$group_index = ceil($this->col / $special_group_size);
			$this->css_extra_class = "special_group_".$group_index;
			$this->single = TRUE;
		}
		$this->top = 0 - (($this->row - 1) * constant('BLOCK_HEIGHT'));
		$this->left = 0 - (($this->col - 1) * constant('BLOCK_WIDTH'));
		$this->css_class = $this->prefix.$index;
	}
	
	public function can_duplicate(){
		return !$this->single;
	}
	
	private function get_left(){
		return $this->left;
	}
	
	private function get_top(){
		return $this->top;
	}
		
	public function get_name(){
		return $this->css_class;
	}
		
	public function get_css_block(){
		return '.'.$this->get_name().'{background-position: '.
				$this->get_left().'px '.$this->get_top().'px;}';
	}

	public function get_positioned_html_block($params){
		$covered = $params["covered"];
		$pos = $params["pos"];
		$layer = $params["layer"];
		$index = $params["index"];
		$zIndex = $params["zIndex"];
		
		$style = ' style="top:'.$pos["top"].'px;left:'.$pos["left"].
				 'px;z-index:'.$zIndex.';" ';
		$classes = 'block '.$this->get_name().' '.
				   $this->css_extra_class.' index_'.$index.
				   ' layer_'.$layer;
		return "\r".
				'<div class="'.$classes.'"'.$style.'>'.
					'<script type="data">'.
						'{index: '.$index.', layer: '.$layer.
							', block_class: \''.$this->css_class.'\''.
							', group_class: \''.$this->css_extra_class.
							'\', single: '.
							($this->single ? "true" : "false").
							', covered: '.$covered.'}'.
					'</script>'.
					'<img src="images/hovered_tile.png" alt=""/>'.
				'</div>';
	}
	
	public function get_html_block(){
		return "\r".'<div class="block '.$this->get_name().' '.$this->css_extra_class.' {}"></div>';
	}
}

?>
