<?php
class Board
{
		private $css = array();
		private $blocks = array();
		private $layout = null;
		
		function __construct($layout) {
			$blocks = array();
			$s_blocks = array(); // Some blocks are special
		
			// The css for a generic board needs to be the first so that it can be overriden easily
			$this->css[] = ".block{".
				//"border: 1px solid gray;".
				"width: ".(constant('BLOCK_WIDTH'))."px;".
				"height:".(constant('BLOCK_HEIGHT'))."px;".
				"background: url(".constant('TILES_IMAGE').") no-repeat scroll;".
				"background-position: 0px 0px;".
				"position:absolute;".
				"float:none;".
			"}";
			
			// For each card/block we have get its block object and it's css
			// seperate the singles from the normal blocks so they will not be duplicated
			for ( $i = 0; $i < constant('BASE_CARDS'); ++$i ) {
				$block = new Block($i);
				if ( $block->can_duplicate() ){
					$blocks[] = $block;
				} else {
					$s_blocks[] = $block;
				}
		
				$this->css[] = $block->get_css_block();
			}
			//$this->blocks = array_merge($blocks, $s_blocks);
			// We need to have 2 pairs of each block, thats 4 total.
			$this->blocks = array_merge($blocks, $blocks, $blocks, $blocks);
			
			// The special blocks are not duplicated but we still need them
			$this->blocks = array_merge($this->blocks, $s_blocks);
			
			// Shuffle the cards to create the order on the board
			shuffle($this->blocks);
			
			// Get the layout
			$this->layout = new Layout($layout);
		}
		
		function get_blocks(){
			return $this->blocks;
		}
		
		
		function get_size(){
			return count($this->blocks);
		}
		
		function get_block($index){
			return $this->blocks[$index];
		}
		
		function get_css(){
			return implode("\n",$this->css);
		}
		
		function get_html(){
			$cols = $this->layout->get_visible_cols();
			$rows = $this->layout->get_visible_rows();
			$board_width =  $cols * constant("BLOCK_WIDTH");
			$board_height = $rows * constant("BLOCK_HEIGHT");
			$board_html = '<div id="game_board" style="position:relative;width: '
							.$board_width.'px;height:'.$board_height.'px;">';
			$board_html .= '<script type="data">{rows: '.$this->layout->get_rows().', '.
							'cols:'.$this->layout->get_cols().
							', layers: '.$this->layout->num_of_layers().'}</script>';
			$layer = 0; // The first "layer"
			$blocks_counter = 0;
			for ( $layer=0, $num_of_layers=$this->layout->num_of_layers();
					$layer < $num_of_layers ; ++$layer){
			
				$active_blocks = $this->layout->get_active_coords_by_layer($layer);		
				// For each block in this layer
				for ($i=0,$total=count($active_blocks);$i<$total;++$i,++$blocks_counter){
					$block = $this->get_block($blocks_counter);
					$params = array("pos"=>$active_blocks[$i]["pos"],
									"index"=>$active_blocks[$i]["index"],
									"layer"=>$layer+1,
									"zIndex"=>$active_blocks[$i]["zIndex"],
									"covered"=>$active_blocks[$i]["covered"]);
					$board_html .= $block->get_positioned_html_block($params);
				}
			}
			$board_html .= '</div>';
			
			print $board_html;
		}
	}
?>
