$(document).ready(function(){
	// Set the metadata type so we can read metadata from a nest script tag which is valid markup
	$.metadata.setType("elem", "script");
	
	// Attach the events
	$(".block").bind("click", toggleBlock);
	$(".block").hover(mouseInBlock, mouseLeaveBlock);
	
	$(document).bind("found_match", foundMatchingBlocks);
	$(document).bind("not_a_match", clearMarkedBlocks);
	$(document).bind("game_won", gameWon);
	$(document).bind("game_over", gameOver);
	$(document).bind("changed_num_blocks", updateFreeBlocks);
	
	$("#start_again").bind("click", reload);
	
	// Some sounds
	$(document).bind("game_won.sound", playCheer);
	$(document).bind("flipped_block.sound", playFlip);
	$(document).bind("found_match", playMatch);
	
	$(document).trigger("changed_num_blocks");
	
	// v2 features
	$("#show_hint").bind("click", showHint);
	$("#shuffle, #game_over_shuffle").bind("click", shuffleBoard);
});

/*
 * Show a hint
 */
function showHint(event){
	event.preventDefault();
	// Get the free blocks
	var $blocks = $("#game_board .block.free");
	var foundMatch = false;
	for(var x=0,total=$blocks.length;
		x<total && !foundMatch;
		++x){
		$block = $($blocks.get(x));
		var class_name = ($block.metadata()["single"] ?
								 $block.metadata()["group_class"]									
								: $block.metadata()["block_class"]);
		// check if this block class has two atleast 2 free blocks
		$match = $("#game_board .block.free."+class_name);
		if ( $match.length > 1 ){
			$match.filter(":first,:last").addClass("hint");
			foundMatch = true;
		}
		$block = $match = null;
	}
	if ( !foundMatch ) {
		alert("There are no more available moves, try to shuffle the board");
	}
	$blocks = null;
	return false;
};

/*
 * Shuffle the board
 */
function shuffleBoard(event){
	event.preventDefault();
	var spots = [];
	
	// Get all the blocks
	var $blocks = $("#game_board .block");
	$.each($blocks, function(index, el){
		// save the used spot so we will keep the layout
		$block = $(el);
		var layer = $block.metadata()['layer'];
		var index = $block.metadata()['index'];
		spots.push({css: {
			top: $block.css("top"),
			left: $block.css("left"),
			"z-index": $block.css("z-index")
		}, layer: layer, index: index, covered: $block.metadata()['covered']});
		$block.removeClass('layer_'+layer).removeClass('index_'+index);
		$block = null;
	});
	// Now shuffle the spots and return the blocks
	spots = shuffle(spots);
	// Place the blocks again
	$.each($blocks, function(index, el){
		$block = $(el);
		var spotInfo = spots.pop();
		$block.css(spotInfo.css);
		// alter the metadata saved for the block to fit with the new spot
		var mdata = $block.metadata();
		mdata['index'] = spotInfo.index;
		mdata['layer'] = spotInfo.layer;
		mdata['covered'] = spotInfo.covered;
		$block.data("metadata", mdata);
		$block.addClass('layer_'+spotInfo.layer).addClass('index_'+spotInfo.index);
		$block = null;
	});
	
	// We need to check for free blocks again since we changed everything
	$blocks.removeClass("free").removeClass("hint");
	$(document).trigger("changed_num_blocks");
	
	$blocks = null;
	return false;
}

function shuffle(v){
    for(var j, x, i = v.length; i; j = parseInt(Math.random() * i), x = v[--i], v[i] = v[j], v[j] = x);
    return v;
};

function updateFreeBlocks(){
	$("#availableMoves").text("...");
	setTimeout(function(){
		var free = findFreeBlocks();
		$("#availableMoves").text(free);
		if ( !free ){
			$(document).trigger("game_over");
		} else {
			gameContinue();
		}
	}, 1);	
};

function mouseInBlock(){
	var $block = $(this);
	if ( $block.is(":not(.selected)") ){
		if ( freeToMove($block) )
			$block.find("img").show();
	}
	$block = null;
};

function mouseLeaveBlock(){
	var $block = $(this);
	if ( $block.is(":not(.selected)") ){
		if ( freeToMove($block) )
			$block.find("img").hide();
	}
	$block = null;
};

function toggleBlock(event){
	var $block = $(this);
	// Check if block is free
	var index = $block.metadata()["index"];
	var layer = $block.metadata()["layer"];
	$block.toggleClass("selected");
	// Did we just marked this block?
	if ( $block.is(".selected") ) {
		$(document).trigger("flipped_block");
		// Make sure the block is free for marking
		if ( freeToMove($block) ){
			$block.find("img").attr("src", "images/selected_tile.png");
			// Check how many blocks selected if pair - act
			var $selected = $block.parent("div").find(".selected");
			if ($selected.length == 2){
				// Two blocks are free to move and selected, check if we 
				// have a match
				var block_class = $block.metadata()["single"] ? 
								$block.metadata()["group_class"] : 
								$block.metadata()["block_class"];
											
				var $matched_blocks = $selected.filter("."+block_class);
				if ( $matched_blocks.length == 2 ){
					$(document).trigger("found_match", 
									{elements: $matched_blocks});
				} else {
					$(document).trigger("not_a_match", 
									{elements: $selected});
				}
				var $matched_cards = null;
			}			
			$selected = null;
		} else {
			$block.toggleClass("selected");
			$(document).trigger("blockNotFree", {element: $block});
		}
	} else {
		$block.find("img").attr("src", "images/hovered_tile.png");
	}
	
	$block = null;
};

function gameWon(){	
	var $game_board = $("#game_board_container");
	var $player_won = $("#player_won");
	$game_board.hide();
	$player_won.show();
	$("#start_again").show();
	$("#game_over_shuffle").hide();
	$game_board = $player_won = null;
};

function gameContinue(){
	var $game_board = $("#game_board_container");
	var $player_lost = $("#player_lost");
	$player_lost.hide();
	$game_board.show();
	$("#start_again").hide();
	
	$game_board = $player_lost = null;
}

function gameOver(){
	var $game_board = $("#game_board_container");
	var $player_lost = $("#player_lost");
	$game_board.hide();
	$player_lost.show();
	$("#start_again").show();
	
	$game_board = $player_lost = null;
};


function foundMatchingBlocks(event, params){
	var $elements = params.elements;
	$elements.remove();

	// Check if we have blocks left
	if ( $("#game_board div.block").length == 0 ){
		// Game won
		$(document).trigger("game_won", {});
	}
	// Don't halt the progress to make this change
	setTimeout(function(){
		$(document).trigger("changed_num_blocks");
	}, 1);
	$elements = null;
};

function clearMarkedBlocks(event, params){
	params.elements.removeClass("selected").find("img").attr("src", "images/hovered_tile.png").hide();
};

function findFreeBlocks(){
	var $blocks = $("#game_board .block");
	var block_classes = {};
	var pairs = 0;
	$.each($blocks, function(index, el){
		var $block = $(el);
		// Check if the block is free
		if ( freeToMove($block) || $block.is(".free") ){
			$block.addClass("free");
			var class_name = ($block.metadata()["single"] ?
								 $block.metadata()["group_class"]									
								: $block.metadata()["block_class"]);
			// If free set the counter 
			block_classes[class_name] = ++block_classes[class_name] || 1;
			if ( block_classes[class_name] == 2 ){
				++pairs;
				block_classes[class_name] = null;
				delete block_classes[class_name];
			}
		}
		$block = null;
	});

	$blocks = null;
	return pairs;
}

function freeToMove(el){
	var isFree = false;
	// If the element was marked free it can't be changed
	if ( el.is(".free") ){ 
		return true;
	}
	var index = el.metadata()["index"];
	var layer = el.metadata()["layer"];
	
	// Check the same layer
	// Something on the left
	var $parent = el.parent("div#game_board");
	var cols = $parent.metadata()["cols"];
	var current_col = (index+1)%cols;
	var same_layer_class = ".layer_"+layer;
	var left_side_class = false;
	var right_side_class = false;
	if ( current_col != 1 ){
		left_side_class = "div.index_"+(index-2)+same_layer_class;
		left_side_class += ",div.index_"+(index-2-cols)+same_layer_class;
		left_side_class += ",div.index_"+(index-2+cols)+same_layer_class;
	} 
	// Make sure there is room for a full block or dont check it
	if ( current_col < (cols - 1) ){
		right_side_class = "div.index_"+(index+2)+same_layer_class;
		right_side_class += ",div.index_"+(index+2-cols)+same_layer_class;
		right_side_class += ",div.index_"+(index+2+cols)+same_layer_class;
	}
	
	if (( $parent.find(left_side_class).length == 0 ) ||
		( $parent.find(right_side_class).length == 0 )){
			/* 
			 * Nothing found on the right side or on the left side, 
			 * so the element is free from the sides...
			 * But is it free from above?  
			 * The server saved us some work by telling us if the block is
			 * covered or not.
			 */
			if ( el.metadata()["covered"] ){
				var upper_layer_class = ".layer_"+(layer+1);
				// The blocks above can be in 0.5 distance apart so we need to check a few classes
				// half before, the same, half after and the rows...
			
				var upper_classes = "div.index_"+(index-1)+upper_layer_class; // a bit to the left
				upper_classes += ",div.index_"+(index)+upper_layer_class; // directory above
				upper_classes += ",div.index_"+(index+1)+upper_layer_class; // a bit to the right
				upper_classes += ",div.index_"+(index-cols)+upper_layer_class; // a bit above
				upper_classes += ",div.index_"+(index+cols)+upper_layer_class; // a bit below
				upper_classes += ",div.index_"+(index-1-cols)+upper_layer_class; // a bit above to the left
				upper_classes += ",div.index_"+(index-1+cols)+upper_layer_class; // a bit below to the left
				upper_classes += ",div.index_"+(index+1-cols)+upper_layer_class; // a bit above to the right
				upper_classes += ",div.index_"+(index+1+cols)+upper_layer_class; // a bit below to the right
				// Even one of those blocks will prevent us from taking the block out
				if ( $parent.find(upper_classes).length == 0 ){
					isFree = true;
				}
			} else {
				// Not covered we are ok
				isFree = true;
			}
	}
	$parent = null;
	return isFree;
}

function reload(event){
	event.preventDefault();
	document.location.reload();
	return false;
};

function getFlashMovieObject(movieName){
	if (window.document[movieName]){
		return window.document[movieName];
	}
	if (navigator.appName.indexOf("Microsoft Internet")==-1){
		if (document.embeds && document.embeds[movieName])
			return document.embeds[movieName];
	} else {
		return document.getElementById(movieName);
	}
};

function playCheer(){
	var sfxMovie=getFlashMovieObject("sfx_movie");
	try{
		sfxMovie.cheer();
	} catch(e) {};
};

function playMatch(){	
	var sfxMovie=getFlashMovieObject("sfx_movie");
	try{
		sfxMovie.flip();
	} catch(e) {};
};

function playFlip(){
	var sfxMovie=getFlashMovieObject("sfx_movie");
	try{
		sfxMovie.match();
	} catch(e) {};
};
