<?php
// Turn off all error reporting
error_reporting(0);
// Report all PHP errors (see changelog)
//error_reporting(E_ALL);
//error_reporting(E_ALL & ~E_NOTICE);

class Layout
{
	private $file = "";
	private $map = array();
	private $loaded = false;
	private $rows = 0;
	private $cols = 0;
		
	function __construct($layout_name) {
		$this->file = 
			"layouts".DIRECTORY_SEPARATOR.$layout_name.".inc.php";
		$this->loaded = $this->load_layout_from_file();
	}
	
	public function is_loaded(){
		return $this->loaded;
	}
	
	private function load_layout_from_file(){
		if ( file_exists($this->file) ){
			include $this->file;
			$this->map = $layout;
			$this->rows = $rows;
			$this->cols = $cols;
			return is_array($this->map);
		} else {
			return FALSE;
		}
	}
	
	public function get_rows(){
		return $this->rows;
	}
	
	public function get_cols(){
		return $this->cols;
	}
	
	public function get_visible_rows(){
		return $this->rows / 2;
	}
	
	public function get_visible_cols(){
		return $this->cols / 2;
	}
	
	public function num_of_layers(){
		return count($this->map);
	}
	
	public function is_block_spot($layer, $index){
		return ($this->map[$layer][$index]>0) ? TRUE : FALSE;
	}
	
	private function check_has_block($var){
		return($var>0);
	}
	
	private function get_active_indexes_by_layer($layer){
		return array_keys(array_filter($this->map[$layer], array($this, "check_has_block")));
	}
	
	public function get_active_coords_by_layer($layer){
		$active = $this->get_active_indexes_by_layer($layer);
		$coords = array();
		for($i=0,$total=count($active);$i < $total;++$i){
			$coords[$i] = $this->get_block_coords($active[$i]);
			// Change the location to give a bit of a 3d look
			$coords[$i]["pos"]["top"] = $coords[$i]["pos"]["top"] - ($layer * 6);
			$coords[$i]["pos"]["left"] = $coords[$i]["pos"]["left"] - ($layer * 6);
			$coords[$i]["layer"] = $layer;
			$coords[$i]["index"] = $active[$i];
			$coords[$i]["zIndex"] = $this->map[$layer][$active[$i]];
			// Save some of the clients work since we are faster and see if this block is covered.
			// Indexes that might get in the way in upper layers are 
			// directly above, slight to the left, slight to the right
			$covered = false;
			if ($this->is_block_spot($layer+1, $active[$i]) ||
				$this->is_block_spot($layer+1, $active[$i]-1) || 
				$this->is_block_spot($layer+1, $active[$i]+1) ||
				$this->is_block_spot($layer+1, $active[$i]+$this->cols) ||
				$this->is_block_spot($layer+1, $active[$i]-$this->cols) ||
				$this->is_block_spot($layer+1, $active[$i]-1+$this->cols) ||
				$this->is_block_spot($layer+1, $active[$i]+1+$this->cols) ||
				$this->is_block_spot($layer+1, $active[$i]-1-$this->cols) ||
				$this->is_block_spot($layer+1, $active[$i]+1-$this->cols)){
				$covered = true;
			}
			$coords[$i]["covered"] = ($covered) ? 'true' : 'false';
		}
		
		return $coords;
	}
	
	public function get_block_coords($index){
		++$index; // Easier to calc
		$b_width = constant("BLOCK_WIDTH");
		$b_height = constant("BLOCK_HEIGHT");
		
		// Find the index col
		$col = ($index % $this->cols) ? $index % $this->cols : $this->cols;
		
		// Find the index row
		$row = ceil($index / $this->cols);
		
		// Pieces on the board can be 0.5 apart
		$top = (($row - 1) * (constant('BLOCK_HEIGHT') / 2))-(($row-1)*2);
		$left = (($col - 1) * (constant('BLOCK_WIDTH') / 2))-(($col-1)*2);
		
		//print "<br />The index was: $index The row is: $row, The col is: $col, found left: $left, top: $top";
		return array("pos"=>array("top"=>$top, "left"=>$left));
	}
}
?>
